@echo off

rem arguments: set 1-HOSTNAME, 2-PORT, 3-SID, 4-TNSNAMES, 5-SYSTEM_PASSWORD, 6-D4OSYS_PASSWORD, 7-DEFAULT_TABLESPACE, 8-TEMPORARY_TABLESPACE, 9-SCRIPT_LOCATION

echo \*\*\*
echo \*\*\*
echo \*\*\* This script makes the following assumptions:
echo \*\*\* 1. The script is being executed on the Business Intelligence middle tier server.
echo \*\*\* 2. ORACLE_HOME environment variable is set to the BI Home (Not DB Home).
echo \*\*\* 3. The bi_user has not been previously installed.
echo \*\*\* 4. Oracle Command Line and Discoverer For OLAP has been installed.
echo \*\*\*
echo.
echo.

:: ////////////////////////////////
:main
goto validateargs
:main1
:: *** Check ORACLE_HOME has been set
echo Validating ORACLE_HOME is set.
goto validateoracleenv

:main2
:: *** Determine whether the OLAP option is installed
echo Validating if OLAP components are installed in the target database
goto validateolap
:main3
:: *** Check bi_user does not exist
echo Validating bi_user does not exist ...
goto bi_userexists
:main4
:: *** Check the SYSTEM password ...
echo Validating SYSTEM password ...
goto validatesystempassword
:main5
:: *** Check the location of the BI_HOME
echo Validating the location of BI_HOME exists ...
goto validatebihome
:main6
:: *** Check for EUL Command line
echo Validating EUL Command Line installed - %BI_HOME%\bin\eulbuilder.jar ...
goto validateeulbuilder
:main7
:: *** Check for D4O
echo Validating D4O installed - %BI_HOME%/sysman/webapps/emd/WEB-INF/lib/d4o.jar ...
if %OLAP_INSTALLED%=="TRUE" goto validated4oinstalled
:main8
:: *** Check tablespaces exist
echo Validating tablespaces ...
goto validatetablespace
:main9
:: *** Check the location of the script
echo Validating the script location ...
goto validatescriptlocation
:main10
:: *** Check the Host, Port, SID
:: echo Validating Host, Port, SID
:: goto validatehostportsid
:main11

:: ////////////////////////////////
:: *** Invoke EUL command line and create bi_user
echo Creating bi_user through EUL Command Line.
cd %SCRIPT_LOCATION%\disco
call mtdisco.bat %TNSNAMES% %SYSTEM_PASSWORD% %BI_HOME% %DEFAULT_TABLESPACE% %TEMPORARY_TABLESPACE% %SCRIPT_LOCATION%\disco
if %OLAP_INSTALLED%=="TRUE" goto callolapscript
if %OLAP_INSTALLED%=="FALSE" goto warningolapnotinstalled
:main12
goto end
:endmain

:: ////////////////////////////////
:callolapscript
echo *********************
echo Create Disco catalog ...
cd %SCRIPT_LOCATION%\olap
call mtolap.bat %HOSTNAME% %PORT% %SID% %SYSTEM_PASSWORD% %BI_HOME% %D4OSYS_PASSWORD% %SCRIPT_LOCATION%\olap %TNSNAMES%
cd %SCRIPT_LOCATION%
goto end

:: ////////////////////////////////
:validateolap
echo 
echo Connecting to SQL*Plus using as system user
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @check_olap.sql
findstr "OLAP Analytic Workspace" check_olap.log > nul
if ERRORLEVEL 1 goto errorolapaws
findstr "Oracle OLAP API" check_olap.log > nul
if ERRORLEVEL 1 goto errorolapapi
findstr "OLAP Catalog" check_olap.log > nul
if ERRORLEVEL 1 goto errorolapcatalog
echo OLAP Components are present in the target database.
set OLAP_INSTALLED="TRUE"
goto main3

:: ////////////////////////////////
:bi_userexists
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @user_exists.sql BI_USER
if ERRORLEVEL 1 goto main4
goto errorbi_userexists

:: ////////////////////////////////
:errorbi_userexists
echo \*\*\*
echo \*\*\* Error: bi_user already exists.
echo \*\*\* Please drop the bi_user if a previous installation failed.
echo \*\*\*
goto end

:: ////////////////////////////////
:validatehostportsid
set TNSNAME_CHECK=(DESCRIPTION=(ADDRESS_LIST=(ADDRESS=(PROTOCOL = TCP)(HOST=%HOSTNAME%)(PORT=%PORT%)))(CONNECT_DATA=(SERVICE_NAME=%ORACLE_SID%)))
echo sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAME_CHECK% @logon.sql
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAME_CHECK% @logon.sql
if ERRORLEVEL 1 goto invalidhostportsid
goto main10

:: ////////////////////////////////
:validatetablespace
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @tablespace.sql %DEFAULT_TABLESPACE%
if ERRORLEVEL 1 goto errordefaulttablespace
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @tablespace.sql %TEMPORARY_TABLESPACE%
if ERRORLEVEL 1 goto errortemporarytablespace
goto main9

:: ////////////////////////////////
:validatescriptlocation
if NOT EXIST %SCRIPT_LOCATION%/install_all.bat goto invalidscriptlocation
goto main10

:: ////////////////////////////////
:validated4oinstalled
if NOT EXIST %BI_HOME%/sysman/webapps/emd/WEB-INF/lib/d4o.jar goto invalidd4o
goto main8

:: ////////////////////////////////
:validateeulbuilder
if NOT EXIST %BI_HOME%\bin\eulbuilder.jar goto invalidcommandline
goto main7

:: ////////////////////////////////
:validatebihome
if NOT EXIST %BI_HOME%\bin goto invalidbihome
goto main6

:: ////////////////////////////////
:validatesystempassword
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @logon.sql
if ERRORLEVEL 1 goto invalidsystempassword
goto main5

:: ////////////////////////////////
:validateoracleenv
if "%ORACLE_HOME%"=="" goto oracleenvnotset
echo 
echo ORACLE_HOME environment variable is set.
goto main2

:: ////////////////////////////////
:validateargs
if "%1"=="" goto usage
if "%2"=="" goto usage
if "%3"=="" goto usage
if "%4"=="" goto usage
if "%5"=="" goto usage
if "%6"=="" goto usage
if "%7"=="" goto usage
if "%8"=="" goto usage
if "%9"=="" goto usage
goto setcommonvars
goto main1

:: ////////////////////////////////
:setcommonvars
set HOSTNAME=%1
set PORT=%2
set SID=%3
set TNSNAMES=%4
set SYSTEM_PASSWORD=%5
set D4OSYS_PASSWORD=%6
set DEFAULT_TABLESPACE=%7
set TEMPORARY_TABLESPACE=%8
set SCRIPT_LOCATION=%9
set BI_HOME=%ORACLE_HOME%

:: Display the arguments to the user to verify they are correct
echo ############
echo
echo You have specified the following arguments to install_all.bat:
echo hostname              %HOSTNAME%
echo port                  %PORT%
echo sid                   %SID%
echo tnsnames_entry        %TNSNAMES%
echo system_password       %SYSTEM_PASSWORD%
echo D4OSYS_password       %D4OSYS_PASSWORD%
echo default_tablespace    %DEFAULT_TABLESPACE%
echo temporary_tablespace  %TEMPORARY_TABLESPACE%
echo location_script       %SCRIPT_LOCATION%
echo Your ORACLE_HOME	   %ORACLE_HOME%
echo
echo ############

echo.
goto main1

:: ////////////////////////////////
:warningolapnotinstalled
echo \*\*\*
echo \*\*\* WARNING: OLAP not setup, some or all of the following
echo \*\*\* OLAP components are missing from the target database:
echo \*\*\* OLAP Anaytic Workspace,
echo \*\*\* Oracle OLAP API,
echo \*\*\* Oracle Catalog.
echo \*\*\*
goto end

:: ////////////////////////////////
:errorolapaws
echo \*\*\*
echo \*\*\* WARNING: OLAP Analytic Workspace component not installed.
echo \*\*\*
set OLAP_INSTALLED="FALSE"
goto main3

:: ////////////////////////////////
:errorolapapi
echo \*\*\*
echo \*\*\* WARNING: Oracle OLAP API component not installed.
echo \*\*\*
set OLAP_INSTALLED="FALSE"
goto main3

:: ////////////////////////////////
:errorolapcatalog
echo \*\*\*
echo \*\*\* WARNING: OLAP Catalog component not installed.
echo \*\*\*
set OLAP_INSTALLED="FALSE"
goto main3

:: ////////////////////////////////
:errorshdoesnotexist
echo \*\*\*
echo \*\*\* Error: SH schema does not exist.
echo \*\*\* Please install the Sales History demo schema.
echo \*\*\*
goto end

:: ////////////////////////////////
:invalidd4o
echo \*\*\*
echo \*\*\* Error: Cannot find D4O - %BI_HOME%/discoverer/lib/d4o.jar.
echo \*\*\* Please ensure D4O is installed, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:errordefaulttablespace
echo \*\*\*
echo \*\*\* Error: The tablespace %DEFAULT_TABLESPACE% does not exist in the database.
echo \*\*\* Please ensure you supply a valid default tablespace, and try again.
echo \*\*\*

:: ////////////////////////////////
:errortemporarytablespace
echo \*\*\*
echo \*\*\* Error: The tablespace %TEMPORARY_TABLESPACE% does not exist in the database.
echo \*\*\* Please ensure you supply a valid temporary tablespace, and try again.
echo \*\*\*

:: ////////////////////////////////
:invalidcommandline
echo \*\*\*
echo \*\*\* Error: Cannot find EUL Command line - %BI_HOME%\bin\eulbuilder.jar.
echo \*\*\* Please ensure EUL Command line is installed, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:invalidbihome
echo \*\*\*
echo \*\*\* Error: Invalid location for BI Home.
echo \*\*\* Please ensure the location of the BI Home is valid, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:invalidsystempassword
echo \*\*\*
echo \*\*\* Error: Cannot authenticate using supplied SYSTEM credentials.
echo \*\*\* Please validate the supplied password and tnsnames, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:invalidscriptlocation
echo \*\*\*
echo \*\*\* Error: Invalid script location, %SCRIPT_LOCATION%
echo \*\*\* Cannot find the install_all.bat within specified script location.
echo \*\*\* Please ensure the script location is correct, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:invalidhostportsid
echo \*\*\*
echo \*\*\* Error: Cannot logon using the Host, Port, SID supplied.
echo \*\*\* Please ensure the supplied Host, Port and ORACLE_SID environment
echo \*\*\* variable are correctly set, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:oracleenvnotset
echo.
echo.
echo - Make sure your ORACLE_HOME is correctly set
echo - The ORACLE_HOME must be set to the BI Home and not the DB Home.
echo -
echo - for example SET ORACLE_HOME=d:\oracle\BIHome
echo.
goto end

:: ////////////////////////////////
:usage
echo Usage: install hostname port sid tnsnames_entry system_password D4OSYS_password default_tablespace temporary_tablespace
echo   hostname             The database host name.
echo   port                 The database port.
echo   sid                  The database sid.
echo   tnsnames_entry       The tnsnames entry for the database server.
echo   system_password      The SYSTEM password.
echo   D4OSYS_password      The D4OSYS password.
echo   default_tablespace   The default tablespace for bi_user, in UPPER CASE.
echo   temporary_tablespace The temporary tablespace for bi_user, in UPPER CASE.
echo   location_script      Fully qualified location of the script.
echo.
echo For example:
echo install testmachine 1521 10G ora10g manager d4osys USERS TEMP d:\scripts
echo.
goto end

:: ////////////////////////////////
:end
