@echo off

rem arguments: set 1-TNSNAMES, 2-SYSTEM_PASSWORD, 3-DEFAULT_TABLESPACE, 4-TEMPORARY_TABLESPACE, 5-SCRIPT_LOCATION

echo \*\*\*
echo \*\*\*
echo \*\*\* This script makes the following assumptions:
echo \*\*\* 1. The script is being executed on the Business Intelligence middle tier server.
echo \*\*\* 2. ORACLE_HOME environment variable is set to the BI Home (Not the database Home).
echo \*\*\* 3. The bi_user has not been previously installed, or it has been removed if installed.
echo \*\*\* 4. Command Line for Discoverer has been installed
echo \*\*\*
echo.
echo.

:: ////////////////////////////////
:main
goto validateargs

:main1
:: *** Check ORACLE_HOME has been set
echo Validating ORACLE_HOME is set.
goto validateoracleenv

:main3
:: *** Check bi_user does not exist. If it does, throw an error and exit.
echo Validating bi_user does not exist ...
goto bi_userexists

:main4
:: *** Check the SYSTEM password ...
echo Validating SYSTEM password ...
goto validatesystempassword

:main5
:: *** Check the location of the BI_HOME
echo Validating the location of BI_HOME exists ...
goto validatebihome

:main6
:: *** Check for EUL Command line
echo Validating EUL Command Line installed - %BI_HOME%\bin\eulbuilder.jar ...
goto validateeulbuilder

:main8
:: *** Check tablespaces exist
echo Validating tablespaces ...
goto validatetablespace

:main9
:: *** Check the location of the script
echo Validating the script location ...
goto validatescriptlocation

:main10

:main11

:: ////////////////////////////////
:: *** Invoke EUL command line and create bi_user
echo Creating bi_user through EUL Command Line.
cd disco
call mtdisco.bat %TNSNAMES% %SYSTEM_PASSWORD% %BI_HOME% %DEFAULT_TABLESPACE% %TEMPORARY_TABLESPACE% %SCRIPT_LOCATION%\disco
cd..

:main12
goto end
:endmain

:: ////////////////////////////////
:: I F   B I   U S E R   E X I S T S
:bi_userexists
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @user_exists.sql BI_USER
if ERRORLEVEL 1 goto main4
goto errorbi_userexists

:: ////////////////////////////////
:: B I   U S E R   E X I S T S
:errorbi_userexists
echo \*\*\*
echo \*\*\* Error: bi_user already exists.
echo \*\*\* Please drop the bi_user if a previous installation failed.
echo \*\*\*
goto end


:: ////////////////////////////////
:: V A L I D A T E   T A B L E S P A C E
:validatetablespace
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @tablespace.sql %DEFAULT_TABLESPACE%
if ERRORLEVEL 1 goto errordefaulttablespace
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @tablespace.sql %TEMPORARY_TABLESPACE%
if ERRORLEVEL 1 goto errortemporarytablespace
goto main9

:: ////////////////////////////////
:: V A L I D A T E   S C R I P T   L O C A T I O N 
:validatescriptlocation
if NOT EXIST %SCRIPT_LOCATION%/install_rel.bat goto invalidscriptlocation
goto main10


:: ////////////////////////////////
::  V A L I D A T E   E U L B U I L D E R . J A R 
:validateeulbuilder
if NOT EXIST %BI_HOME%\bin\eulbuilder.jar goto invalidcommandline
goto main8

:: ////////////////////////////////
:: V A L I D A T E   B I   H O M E
:validatebihome
if NOT EXIST %BI_HOME%\bin goto invalidbihome
goto main6

:: ////////////////////////////////
:: V A L I D A T E   S Y S T E M   P A S S W O R D
:validatesystempassword
%ORACLE_HOME%\bin\sqlplus -L -S system/%SYSTEM_PASSWORD%@%TNSNAMES% @logon.sql
if ERRORLEVEL 1 goto invalidsystempassword
goto main5

:: ////////////////////////////////
:: V A L I D A T E   E N V   V A R I A B L E
:validateoracleenv
if "%ORACLE_HOME%"=="" goto oracleenvnotset
echo 
echo ORACLE_HOME environment variable is set.
goto main3

:: ////////////////////////////////
:: V A L I D A T E   A R G U M E N T S
:validateargs
if "%1"=="" goto usage
if "%2"=="" goto usage
if "%3"=="" goto usage
if "%4"=="" goto usage
if "%5"=="" goto usage
goto setcommonvars
goto main1

:: ////////////////////////////////
:: S E T   V A R I A B L E S
:setcommonvars
set TNSNAMES=%1
set SYSTEM_PASSWORD=%2
set DEFAULT_TABLESPACE=%3
set TEMPORARY_TABLESPACE=%4
set SCRIPT_LOCATION=%5
set BI_HOME=%ORACLE_HOME%

goto main1

:: ////////////////////////////////
:: S H   S C H E M A   D O E S   N O T   E X I S T
:errorshdoesnotexist
echo \*\*\*
echo \*\*\* Error: SH schema does not exist.
echo \*\*\* Please install the Sales History demo schema. Run the common schema installation script and then try again.
echo \*\*\*
goto end


:: ////////////////////////////////
:: I N V A L I D   D E F A U L  T   T A B L E S P A C E
:errordefaulttablespace
echo \*\*\*
echo \*\*\* Error: The tablespace %DEFAULT_TABLESPACE% does not exist in the database.
echo \*\*\* Please ensure you supply a valid default tablespace, and try again.
echo \*\*\*

:: ////////////////////////////////
:: I N V A L I D   T E M P   T A B L E S P A C E
:errortemporarytablespace
echo \*\*\*
echo \*\*\* Error: The tablespace %TEMPORARY_TABLESPACE% does not exist in the database.
echo \*\*\* Please ensure you supply a valid temporary tablespace, and try again.
echo \*\*\*

:: ////////////////////////////////
:: I N V A L I D   B I   H O M E
:invalidbihome
echo \*\*\*
echo \*\*\* Error: Invalid location for BI Home.
echo \*\*\* Please ensure the location of the BI Home is valid, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:: I N V A L I D   S Y S T E M   P A S S W O R D
:invalidsystempassword
echo \*\*\*
echo \*\*\* Error: Cannot authenticate using supplied SYSTEM credentials.
echo \*\*\* Please validate the supplied password and tnsnames, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:: I N V A L I D   L O C A T I O N   S P E C I F I E D   F O R   T H E   S C R I P T
:invalidscriptlocation
echo \*\*\*
echo \*\*\* Error: Invalid script location, %SCRIPT_LOCATION%
echo \*\*\* Cannot find the install_rel.bat within specified script location.
echo \*\*\* Please ensure the script location is correct, and try again.
echo \*\*\*
goto end

:: ////////////////////////////////
:: O R A C L E   H O M E   N O T   S E T
:oracleenvnotset
echo.
echo.
echo - Make sure your ORACLE_HOME is correctly set
echo - The ORACLE_HOME must be set to the BI Home and not the DB Home.
echo -
echo - for example SET ORACLE_HOME=d:\oracle\BIHome
echo.
goto end

:: ////////////////////////////////
:: S H O W   U S A G E
:usage
echo Usage: install tnsnames_entry system_password default_tablespace temporary_tablespace
echo   tnsnames_entry       The tnsnames entry for the database server.
echo   system_password      The SYSTEM password.
echo   default_tablespace   The default tablespace for bi_user, in UPPER CASE.
echo   temporary_tablespace The temporary tablespace for bi_user, in UPPER CASE.
echo   location_script      Fully qualified location of the script.
echo.
echo For example:
echo install ora10g manager USERS TEMP d:\scripts
echo.
goto end

:: ////////////////////////////////
:end
