#!/bin/ksh

export PATH=$BI_HOME/jdk/bin:$PATH

if [ $# -lt 6 ]
then
	echo Please specify the following four parameters when running the install:
	echo "hostname	- name of the host machine where the database resides"
	echo "port 		- database port"
	echo "SID		- database SID"
	echo "system_password	- password for SYSTEM user"
	echo "d4osys_password	- password for the new D4OSYS user (owner of the Discoverer Catalog)"
	echo "OracleBI home	- Oracle Business Intelligence Home"
	echo "Net Service Name (required for remote database connections)"
	echo
	echo "Note - your PATH should include the location of the Java Runtime (i.e. java.exe) and SQL*Plus"
	echo
	echo For example:
	echo install_olap localhost 1521 orcl system_pwd d4osys_pwd c:\OraBIHome remoteserver
	exit 1
fi

HOST=$1
PORT=$2
SID=$3
SYSTEM_PASSWD=$4
D4OSYS_PASSWD=$5
BI_HOME=$6
NET_SERVICE_NAME=$7

if [ $NET_SERVICE_NAME = "" ]
then
	$BI_HOME/bin/sqlplus system/$SYSTEM_PASSWD @check_for_catalog.sql
else
	$BI_HOME/bin/sqlplus system/$SYSTEM_PASSWD@$NET_SERVICE_NAME @check_for_catalog.sql
fi	

grep "D4OSYS" CATALOG.LOG > /dev/null

if [ $? -ne 0 ]
then
java -classpath $BI_HOME/sysman/webapps/emd/WEB-INF/lib/d4o.jar oracle.dss.d4o.administration.D4OCommand install -su system -sp $SYSTEM_PASSWD -p $D4OSYS_PASSWD -h $HOST -po $PORT -sid $SID
else
echo "Autorizing user SCOTT to access common schema data"
if [ $NET_SERVICE_NAME = "" ]
then
	$BI_HOME/bin/sqlplus system/$SYSTEM_PASSWD @authorize.sql
else
	$BI_HOME/bin/sqlplus system/$SYSTEM_PASSWD@$NET_SERVICE_NAME @authorize.sql
fi
fi

#Add SCOTT user to the Discoverer Catalog
echo "Authorizing SCOTT to use Discoverer"
java -classpath $BI_HOME/sysman/webapps/emd/WEB-INF/lib/d4o.jar oracle.dss.d4o.administration.D4OCommand authorize -p $D4OSYS_PASSWD -h $HOST -po $PORT -sid $SID -u SCOTT

#Add BI_USER  user to the Discoverer Catalog
echo "Authorizing BI_USER to use Discoverer"
java -classpath $BI_HOME/sysman/webapps/emd/WEB-INF/lib/d4o.jar oracle.dss.d4o.administration.D4OCommand authorize -p $D4OSYS_PASSWD -h $HOST -po $PORT -sid $SID -u BI_USER

#IMPORT the sample workbooks, calculations and saved selections for SCOTT
echo "Import the sample workbooks, calculations and saved selections for SCOTT"
java -classpath $BI_HOME/sysman/webapps/emd/WEB-INF/lib/d4o.jar oracle.dss.d4o.administration.D4OCommand import -p $D4OSYS_PASSWD -h $HOST -po $PORT -sid $SID -u SCOTT -f ./scott_catalog.xml

#Install the d4o_autogo which will scope the time dimension for user SCOTT
if [ $NET_SERVICE_NAME = "" ]
then
	$BI_HOME/bin/sqlplus system/$SYSTEM_PASSWD @d4o_autogo.sql
else
	$BI_HOME/bin/sqlplus system/$SYSTEM_PASSWD@$NET_SERVICE_NAME @d4o_autogo.sql
fi

echo Installation of sample complete.
